@@ -96,22 +96,22 @@ def plot_cmap(data):
     plt.show()
     plt.pause(.001)
 
-def plot_confusion(c, actuals, preds):
+def calc_cm(c, actuals, preds):
     cm = np.zeros((c, c), dtype=int)
     cm_p = np.zeros((c, c), dtype=np.float)
     for i in range(preds.shape[0]): cm[int(actuals[i])][int(preds[i])] += 1
     for i in range(cm.shape[0]): cm_p[i,:] = cm[i,:] / cm[i,:].sum()
     cm_p = 100.0*cm_p
+    return cm,cm_p
+
+def plot_confusion(cm, cm_p):
     fig = plt.figure(figsize=(20, 5))
     ax1, ax2 = fig.add_subplot(121), fig.add_subplot(122)
     sns.set(font_scale=1.0)
     sns.heatmap(cm, cmap=mpl.cm.Blues, annot=True, fmt='d', ax=ax1)
     sns.heatmap(cm_p, cmap=mpl.cm.Blues, annot=True, fmt='.2f', ax=ax2)
-    ax1.set_title(f'Total: {preds.shape[0]}')
-    ax2.set_title(f'Total: 100.00%')
     plt.show()
     plt.pause(.001)
-    return cm
 
 def calc_prc(y, y2):
     mask = np.argsort(-y2)
@@ -189,71 +189,39 @@ def get_predictions_clsf_tabular(data, model, df, device='cpu'):
     df['pred_label'] = [data.cls[e] for e in pred]
     return df
 
-
-# ???
-def get_probs_for_actuals(y2, actuals):
-    probs = np.exp(y2)
-    mask = np.zeros((probs.shape[0], probs.shape[1]))
-    mask[np.arange(actuals.shape[0]),actuals] = 1
-    return probs[mask.astype(bool)]
-
-
-# ???
-def df_predict_clsf(learner, df, have_actuals=False):
-    df = df.copy()
-    if 'prob' in df.columns: df=df.drop(columns='prob')
-    data = learner.data
-    y2 = learner.predict(df)
-    y2 = y2.numpy()
-    prob = get_probs(y2) if data.c>2 else get_probs_bin(y2)
-    pred = get_preds(y2)
-    pred_label = [data.cls[e] for e in pred]
-    df['prob'] = prob
-    df['pred'] = pred
-    df['pred_label'] = pred_label
-    if have_actuals:
-        actuals = df[data.proc.c_dep] 
-        df['actual'] = actuals.values
-        df['actual_label'] = [data.cls[e] for e in actuals.values]
-        df['error'] = (df['pred']!=df['actual'])*1
-    df = df.sort_values(by=['prob'], ascending=[False])
-    return df
-
-
 class Results_Clsf():
-    def __init__(self, probs, actuals):
-        self.probs, self.actuals = probs, actuals
-        self.preds = get_preds(probs)
-        self.c = len(np.unique(actuals))
-#         self.probs_for_actuals = get_probs_for_actuals(y2, self.actuals)
-#         self.probs_sorted = np.argsort(self.probs_for_actuals)[::-1]
-    def show_all(self, thresholds=10):
+    def __init__(self, learner):
+        y2,y = learner.logger.y2_ep.cpu().numpy(), learner.logger.y_ep.cpu().numpy()
+        self.actuals = y
+        self.c = len(np.unique(y))
+        self.probs = get_probs_bin(y2) if self.c==2 else get_probs(y2)
+        self.preds = get_preds(self.probs)
+    def all(self, thresholds=10):
         self.confusion()
         self.roc()
-        display(self.df_stats(thresholds=thresholds))
+        self.stats()
     def confusion(self, t=None):
-        show_dist(self.actuals)
-        display(df_stats_binary(self.preds, self.actuals))
-        if t is None:
-            preds = self.preds
-        else:
-            if self.c!=2: raise ValueError('More than two classes')
-            preds = (self.probs>t)*1
-        cm = plot_confusion(self.c, self.actuals, preds)
-        display(df_stats_by_class(cm))
-    def df_stats(self, thresholds=10):
         if self.c!=2: raise ValueError('More than two classes')
-        return df_stats_cum(self.probs, self.actuals, thresholds=thresholds)
+        cm,cm_p = calc_cm(self.c, self.actuals, self.preds)
+        plot_confusion(cm, cm_p)
+        display(pd.DataFrame(m.sum(axis=0)).transpose())
+        display(pd.DataFrame(m.sum(axis=1)))
+        display(df_stats_by_class(cm))
+    def stats(self, thresholds=10):
+        if self.c!=2: raise ValueError('More than two classes')
+        display(df_stats_binary(self.preds, self.actuals))
+        display(df_stats_cum(self.probs, self.actuals, thresholds=thresholds))
     def roc(self):
         if self.c!=2: raise ValueError('More than two classes')
         plot_roc(self.actuals, self.probs)
-#     def show_best(self, n=8, **kwargs):  self.show_set(self.data.ds2, self.probs_sorted[:n], **kwargs)
-#     def show_worst(self, n=8, **kwargs): self.show_set(self.data.ds2, self.probs_sorted[-n:][::-1], **kwargs)
-#     def show_set(self, ds, idxs, **kwargs):
-#         labels = [f'{ds.labels[i]}:{self.actuals[i]} -> {ds.cls[self.preds[i]]}:{self.preds[i]} (p={self.probs_for_actuals[i]:.4f})'
-#                     for i in idxs]
-#         ds.show(list(idxs), labels=labels, **kwargs)
+    def best(self, n=8, **kwargs): return np.argsort(self.probs)[::-1][:n]
+    def worst(self, n=8, **kwargs): return np.argsort(self.probs)[:n]
 
+# y2,y = learner.logger.y2_ep.cpu().numpy(), learner.logger.y_ep.cpu().numpy()
+# c = len(np.unique(actuals))
+# probs = get_probs_bin(y2) if c==2 else get_probs(y2)
+# actuals = y
+# preds = get_preds(probs)
 
 class Results_Clsf_Tabular(Results_Clsf):
     def __init__(self, data, df):
